-- Player Controller

player = {}
player.pos = {x=0,y=0}

pc = {} 
pc.img = love.graphics.newImage('assets/purple.png')

function pc.load()
	
	-- Physics
	
	pc.body = love.physics.newBody(world, 0, 0, "dynamic")
	pc.shape = love.physics.newRectangleShape(16, 16, 30, 30)
	pc.fixture = love.physics.newFixture(pc.body, pc.shape, 5)
	
	pc.body:setFixedRotation(true)
	pc.fixture:setFriction(0)
	
	-- Ground Check
	
	pc.grndCheck = {}
	pc.grndCheck.shape = love.physics.newRectangleShape(16, 32, 32, 2)
	pc.grndCheck.fixture = love.physics.newFixture(pc.body, pc.grndCheck.shape)
	
	pc.grndCheck.fixture:setSensor(true)
	pc.grndCheck.fixture:setUserData("Ground Check")

	-- Movement

	pc.moveSpeed = 150
	pc.currentSpeed = 0
	
	pc.jumpHeight = 300
	pc.fallMult = 2.5
	pc.lowJMult = 3
	
	-- This controls the amount of time while airborne you can be while still being able to jump,
	--	such as walking off a ledge and jumping half a second after you techinically were off the platform
	pc.coyoteTime = 0.15
	pc.coyoteMeter = 0
	
	pc.startFrictionMultiplier = 20
	pc.endFrictionMultiplier = 20
	pc.maxSpeedPercentage = 0
	
end

function pc.update(dt)

	local movement = vector2.new(0, 0)

	if love.keyboard.isDown('d') then
		movement.x = movement.x + 1
	end
	if love.keyboard.isDown('a') then
		movement.x = movement.x - 1
	end
	if love.keyboard.isDown('w') then
		movement.y = movement.y + 1
	end
	if love.keyboard.isDown('s') then
		movement.y = movement.y - 1
	end
	
	-- Get velocity before movement
	local vel = vector2.new(pc.body:getLinearVelocity())
	
	-- Left, right, none
    if movement.x > 0.3 then
		pc.maxSpeedPercentage = mathf.cap(pc.maxSpeedPercentage + (dt * pc.startFrictionMultiplier), 1)
		pc.currentSpeed = mathf.lerp(0, pc.moveSpeed, pc.maxSpeedPercentage)
        vel = vector2.new(movement.x * pc.currentSpeed, vel.y)
    elseif movement.x < -0.3 then
		pc.maxSpeedPercentage = mathf.bot(pc.maxSpeedPercentage - (dt * pc.startFrictionMultiplier), -1)
		pc.currentSpeed = mathf.lerp(0, pc.moveSpeed, mathf.remap(pc.maxSpeedPercentage, 0, -1, 0, 1))
        vel = vector2.new(movement.x * pc.currentSpeed, vel.y)
	else
		pc.maxSpeedPercentage = mathf.lerp(pc.maxSpeedPercentage, 0, dt * pc.endFrictionMultiplier)
        vel = vector2.new(mathf.lerp(vel.x, 0, dt * pc.endFrictionMultiplier), vel.y)
	end
	
	-- Down/Crouch
	if movement.y < -0.3 then
		
	end
	
	-- Jumping
	
	if vel.y > 0 then
		local v0 = vector2.mult(vector2.up, vector2.new(world:getGravity()))
		local v1 = vector2.mult(v0, (pc.fallMult-1)*dt)
		vel = vector2.add(vel, v1)
	elseif vel.y < 0 and not love.keyboard.isDown('space') then
		local v0 = vector2.mult(vector2.up, vector2.new(world:getGravity()))
		local v1 = vector2.mult(v0, (pc.lowJMult-1)*dt)
		vel = vector2.add(vel, v1)
	end
	
	pc.body:setLinearVelocity(vel.x, vel.y)
	
	if pc.isGrounded() == false then
		if pc.jumping == true then
			pc.coyoteMeter = pc.coyoteTime + 1
		else
			pc.coyoteMeter = pc.coyoteMeter + dt
		end
	else
		pc.coyoteMeter = 0
	end
	
	if love.keyboard.isDown('space') and pc.coyoteMeter < pc.coyoteTime then  
		pc.jump()
	end
	
	player.pos.x, player.pos.y = pc.body:getPosition()
	
	if debug then
		print(pc.isGrounded() and "GRND" or "", pc.coyoteMeter)
	end
end

function pc.draw()
	love.graphics.setColor(1,1,1)
	love.graphics.draw(pc.img, player.pos.x, player.pos.y)
end

function pc.keypressed(key)
	if key == 'space' then
		pc.jump(pc.img, pc.x, pc.y)
	end
end

function pc.jump()
	if pc.coyoteMeter < pc.coyoteTime then
		print("JUMPING "..math.random())
		local x, y = pc.body:getLinearVelocity()
		pc.body:setLinearVelocity(x, -pc.jumpHeight)
		pc.jumping = true
	end
end

function pc.isGrounded()
	for _,v in pairs(pc.body:getContacts()) do
		local fA, fB = v:getFixtures()
		if (fA:getUserData() == "Ground" and fB:getUserData() == "Ground Check") or (fB:getUserData() == "Ground" and fA:getUserData() == "Ground Check") then
			return true
		end
	end
	return false
end

function pc.destroy()
	pc.body:destroy()
	pc.fixture = nil
	pc.shape = nil
	pc.grndCheck = nil
end
